@@ -1,5 +1,8 @@
+# coding: utf-8
 import os
 import sys
+import codecs
+import locale
 
 if sys.version_info < (2, 7):
     import unittest2 as unittest
@@ -1319,10 +1322,14 @@ class TestFixPEP8Warning(unittest.TestCase):
             temp_file.write(line)
         opts, _ = autopep8.parse_args([self.tempfile[1]] + list(options))
         sio = StringIO()
+        if sys.version_info[0] < 3:
+            output = codecs.getwriter(locale.getpreferredencoding())(sio)
+        else:
+            output = sio
         autopep8.fix_file(filename=self.tempfile[1],
                           opts=opts,
-                          output=sio)
-        self.result = sio.getvalue()
+                          output=output)
+        self.result = output.getvalue()
 
     def test_w291(self):
         line = "print 'a b '\t \n"
@@ -1430,6 +1437,18 @@ a.has_key(
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_w601_with_non_ascii(self):
+        line = "## éはe\ncorrect = dict().has_key('good syntax ?')\n"
+        fixed = "## éはe\ncorrect = 'good syntax ?' in dict()\n"
+        if sys.version_info[0] < 3:
+            line = unicode(line, 'utf-8')
+            fixed = unicode(fixed, 'utf-8')
+            self._inner_setup(line.encode('utf-8'))
+            self.assertEqual(self.result, fixed.encode('utf-8'))
+        else:
+            self._inner_setup(line)
+            self.assertEqual(self.result, fixed)
+
     def test_w602_arg_is_string(self):
         line = "raise ValueError, \"w602 test\"\n"
         fixed = "raise ValueError(\"w602 test\")\n"
@@ -1643,7 +1662,7 @@ class TestOptions(unittest.TestCase):
         f = open(self.tempfile[1], 'w')
         f.write(line)
         f.close()
-        p = Popen(list(AUTOPEP8_CMD_TUPLE) + [self.tempfile[1], '--verbose'],
+        p = Popen(list(AUTOPEP8_CMD_TUPLE) + [self.tempfile[1], '-vvv'],
                   stdout=PIPE, stderr=PIPE)
         verbose_error = p.communicate()[1].decode('utf8')
         self.assertIn("'fix_e901' is not defined", verbose_error)
@@ -1654,7 +1673,7 @@ class TestOptions(unittest.TestCase):
         f.write(line)
         f.close()
         p = Popen(list(AUTOPEP8_CMD_TUPLE) +
-                  [self.tempfile[1], '--verbose', '--diff'],
+                  [self.tempfile[1], '-vvv', '--diff'],
                   stdout=PIPE, stderr=PIPE)
         verbose_error = p.communicate()[1].decode('utf8')
         self.assertIn("'fix_e901' is not defined", verbose_error)
